"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeBundles = exports.createBundles = exports.rollupWatch = exports.createRollupConfig = void 0;
const path_1 = require("path");
const fs_extra_1 = __importDefault(require("fs-extra"));
const rollup_1 = require("rollup");
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_commonjs_1 = __importDefault(require("@rollup/plugin-commonjs"));
const plugin_json_1 = __importDefault(require("@rollup/plugin-json"));
const plugin_replace_1 = __importDefault(require("@rollup/plugin-replace"));
const rollup_plugin_typescript2_1 = __importDefault(require("rollup-plugin-typescript2"));
const rollup_plugin_sourcemaps_1 = __importDefault(require("rollup-plugin-sourcemaps"));
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const utils_1 = require("../utils/utils");
const error_1 = require("../utils/error");
const createRollupConfig = ({ tsconfigPath, addUmdBuild, kebabCasePkgName, pkgJsonSideEffects, pkgJsonPeerDependencyKeys, pkgJsonUmdGlobalDependencies, }) => {
    // 默认配置
    const buildPluginsDefault = [
        plugin_node_resolve_1.default({ preferBuiltins: true }),
        plugin_commonjs_1.default(),
        plugin_json_1.default(),
        rollup_plugin_typescript2_1.default({
            tsconfigDefaults: {
                compilerOptions: {
                    target: 'ES2018',
                    types: [],
                    declaration: true,
                    rootDir: './src',
                },
                include: ['src', './node_modules/rollpkg/configs/types'],
                exclude: [
                    '**/__tests__',
                    '**/__mocks__',
                    '**/__snapshots__',
                    '**/*.test.*',
                    '**/*.spec.*',
                    '**/*.mock.*',
                ],
            },
            tsconfig: tsconfigPath,
            tsconfigOverride: {
                compilerOptions: {
                    module: 'ESNext',
                    sourceMap: true,
                },
            },
            include: ['**/*.ts+(|x)', '**/*.js+(|x)'],
        }),
        rollup_plugin_sourcemaps_1.default(),
        plugin_replace_1.default({
            __DEV__: "process.env.NODE_ENV !== 'production'",
            preventAssignment: true,
        }),
    ];
    // 开发配置
    const buildPluginsWithNodeEnvDevelopment = [
        ...buildPluginsDefault,
        plugin_replace_1.default({
            'process.env.NODE_ENV': JSON.stringify('development'),
            preventAssignment: true,
        }),
    ];
    // 生产配置
    const buildPluginsWithNodeEnvProduction = [
        ...buildPluginsDefault,
        plugin_replace_1.default({
            'process.env.NODE_ENV': JSON.stringify('production'),
            preventAssignment: true,
        }),
    ];
    const outputPluginsDefault = [];
    const outputPluginsProduction = [
        rollup_plugin_terser_1.terser({ format: { comments: false } }),
    ];
    const treeshakeOptions = {
        annotations: true,
        moduleSideEffects: pkgJsonSideEffects,
        propertyReadSideEffects: pkgJsonSideEffects,
        tryCatchDeoptimization: pkgJsonSideEffects,
        unknownGlobalSideEffects: pkgJsonSideEffects,
    };
    let umdExternalDependencies;
    let umdDependencyGlobals;
    let umdNameForPkg;
    if (addUmdBuild) {
        umdExternalDependencies = pkgJsonUmdGlobalDependencies
            ? Object.keys(pkgJsonUmdGlobalDependencies)
            : pkgJsonPeerDependencyKeys;
        if (pkgJsonUmdGlobalDependencies) {
            umdDependencyGlobals = pkgJsonUmdGlobalDependencies;
        }
        else {
            pkgJsonPeerDependencyKeys.forEach((peerDep) => {
                umdDependencyGlobals = {};
                umdDependencyGlobals[peerDep] = utils_1.convertKebabCaseToPascalCase(utils_1.convertPkgNameToKebabCase(peerDep));
            });
        }
        umdNameForPkg = utils_1.convertKebabCaseToPascalCase(kebabCasePkgName);
    }
    return {
        buildPluginsDefault,
        buildPluginsWithNodeEnvDevelopment,
        buildPluginsWithNodeEnvProduction,
        outputPluginsDefault,
        outputPluginsProduction,
        treeshakeOptions,
        umdNameForPkg,
        umdExternalDependencies,
        umdDependencyGlobals,
    };
};
exports.createRollupConfig = createRollupConfig;
const rollupWatch = ({ kebabCasePkgName, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, entryFile, treeshakeOptions, buildPluginsDefault, outputPluginsDefault, }) => {
    const exitWatch = () => {
        utils_1.clearLine();
        console.log('WATCH END 👀👋', '\n');
        process.exit(0);
    };
    process.on('SIGINT', exitWatch);
    process.on('SIGTERM', exitWatch);
    process.on('SIGBREAK', exitWatch);
    const esmWatchOptions = {
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsDefault,
        output: {
            file: `dist/${kebabCasePkgName}.esm.js`,
            format: 'esm',
            sourcemap: true,
            plugins: outputPluginsDefault,
        },
    };
    const watcher = rollup_1.watch(esmWatchOptions);
    const earth = ['⚽️', '⚽️', '⚽️'];
    let currentEarth = 2;
    const rotateEarth = () => {
        if (currentEarth === 2)
            currentEarth = 0;
        else {
            currentEarth = currentEarth + 1;
        }
        return earth[currentEarth];
    };
    watcher.on('event', (event) => {
        switch (event.code) {
            case 'START':
                break;
            case 'BUNDLE_START':
                utils_1.clearConsole();
                console.log(`${rotateEarth()} 开始打包...`, '\n');
                break;
            case 'BUNDLE_END':
                utils_1.clearConsole();
                console.log(`${earth[currentEarth]} 打包成功!`, '\n');
                console.log('Watching for changes...', '\n');
                break;
            case 'END':
                break;
            case 'ERROR':
                utils_1.clearConsole();
                if (event.error.plugin === 'rpt2') {
                    error_1.logTsError({ message: event.error.message });
                }
                else {
                    error_1.logError({ fullError: event.error });
                }
                console.log('Watching for changes...', '\n');
                break;
        }
    });
};
exports.rollupWatch = rollupWatch;
const createBundles = ({ entryFile, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, umdExternalDependencies, treeshakeOptions, buildPluginsDefault, buildPluginsWithNodeEnvDevelopment, buildPluginsWithNodeEnvProduction, addUmdBuild, }) => Promise.all([
    // default bundle - used for esm and cjs dev builds
    rollup_1.rollup({
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsDefault,
    }),
    // cjs production build
    rollup_1.rollup({
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsWithNodeEnvProduction,
    }),
    // umd development build
    addUmdBuild
        ? rollup_1.rollup({
            external: umdExternalDependencies,
            input: entryFile,
            treeshake: treeshakeOptions,
            plugins: buildPluginsWithNodeEnvDevelopment,
        })
        : undefined,
    // umd production build
    addUmdBuild
        ? rollup_1.rollup({
            external: umdExternalDependencies,
            input: entryFile,
            treeshake: treeshakeOptions,
            plugins: buildPluginsWithNodeEnvProduction,
        })
        : undefined,
]);
exports.createBundles = createBundles;
const writeBundles = ({ cwd, kebabCasePkgName, bundleDefault, bundleCjsProd, bundleUmdDev, bundleUmdProd, outputPluginsDefault, outputPluginsProduction, umdNameForPkg, umdDependencyGlobals, }) => {
    const cjsEntryContent = `'use strict';

if (process.env.NODE_ENV === 'production') {
  module.exports = require('./${kebabCasePkgName}.cjs.production.js');
} else {
  module.exports = require('./${kebabCasePkgName}.cjs.development.js');
}
`;
    return Promise.all([
        // esm build
        bundleDefault.write({
            file: `dist/${kebabCasePkgName}.esm.js`,
            format: 'esm',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsDefault,
        }),
        // cjs development build
        bundleDefault.write({
            file: `dist/${kebabCasePkgName}.cjs.development.js`,
            format: 'cjs',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsDefault,
        }),
        // cjs production build
        bundleCjsProd.write({
            file: `dist/${kebabCasePkgName}.cjs.production.js`,
            format: 'cjs',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsProduction,
        }),
        // cjs entry file
        fs_extra_1.default.writeFile(path_1.resolve(cwd, 'dist', `${kebabCasePkgName}.cjs.js`), cjsEntryContent, 'utf8'),
        bundleUmdDev
            ? bundleUmdDev.write({
                file: `dist/${kebabCasePkgName}.umd.development.js`,
                format: 'umd',
                name: umdNameForPkg,
                globals: umdDependencyGlobals,
                sourcemap: true,
                sourcemapExcludeSources: false,
                plugins: outputPluginsDefault,
            })
            : undefined,
        bundleUmdProd
            ? bundleUmdProd.write({
                file: `dist/${kebabCasePkgName}.umd.production.js`,
                format: 'umd',
                name: umdNameForPkg,
                globals: umdDependencyGlobals,
                sourcemap: true,
                sourcemapExcludeSources: false,
                plugins: outputPluginsProduction,
            })
            : undefined,
    ]);
};
exports.writeBundles = writeBundles;
//# sourceMappingURL=index.js.map