"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("./utils/error");
const utils_1 = require("./utils/utils");
const config_1 = require("./config");
const mooliRollup = async () => {
    // 清除dist
    const cleanDistMessage = 'Cleaning dist folder';
    try {
        const clean = utils_1.cleanDist();
        await utils_1.progressEstimator(clean, cleanDistMessage);
    }
    catch (error) {
        error_1.logError({
            failedAt: cleanDistMessage,
            message: error_1.errorAsObjectWithMessage(error).message,
            fullError: error,
        });
        throw error_1.EXIT_ON_ERROR;
    }
    // 检查rollup
    const args = process.argv.slice(2);
    const cwd = process.cwd();
    let rollpkgConfiguration;
    try {
        rollpkgConfiguration = utils_1.checkInvariantsAndGetConfiguration({ args, cwd });
    }
    catch (error) {
        throw error_1.EXIT_ON_ERROR;
    }
    const { watchMode, tsconfigPath, addUmdBuild, entryFile, kebabCasePkgName, pkgJsonSideEffects, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, pkgJsonUmdGlobalDependencies, } = await rollpkgConfiguration;
    // rollup配置
    let rollupConfiguration;
    try {
        rollupConfiguration = config_1.createRollupConfig({
            tsconfigPath,
            addUmdBuild,
            kebabCasePkgName,
            pkgJsonSideEffects,
            pkgJsonPeerDependencyKeys,
            pkgJsonUmdGlobalDependencies,
        });
    }
    catch (error) {
        throw error_1.EXIT_ON_ERROR;
    }
    const { buildPluginsDefault, buildPluginsWithNodeEnvDevelopment, buildPluginsWithNodeEnvProduction, outputPluginsDefault, outputPluginsProduction, treeshakeOptions, umdNameForPkg, umdExternalDependencies, umdDependencyGlobals, } = rollupConfiguration;
    // rollup watch
    if (watchMode) {
        config_1.rollupWatch({
            kebabCasePkgName,
            pkgJsonDependencyKeys,
            pkgJsonPeerDependencyKeys,
            entryFile,
            treeshakeOptions,
            buildPluginsDefault,
            outputPluginsDefault,
        });
        return;
    }
    // create rollup bundles
    const createRollupBundlesMessage = `Creating esm, cjs${addUmdBuild ? ', umd' : ''} builds`;
    let bundles;
    try {
        bundles = config_1.createBundles({
            entryFile,
            pkgJsonDependencyKeys,
            pkgJsonPeerDependencyKeys,
            umdExternalDependencies,
            treeshakeOptions,
            buildPluginsDefault,
            buildPluginsWithNodeEnvDevelopment,
            buildPluginsWithNodeEnvProduction,
            addUmdBuild,
        });
        await utils_1.progressEstimator(bundles, createRollupBundlesMessage, {
            id: `${kebabCasePkgName}-${createRollupBundlesMessage}`,
        });
    }
    catch (error) {
        const errorAsObject = error_1.errorAsObjectWithMessage(error);
        if (errorAsObject.plugin === 'rpt2') {
            error_1.logTsError({
                failedAt: createRollupBundlesMessage,
                message: errorAsObject.message,
            });
        }
        else {
            error_1.logError({
                failedAt: createRollupBundlesMessage,
                message: errorAsObject.message,
                fullError: error,
            });
        }
        throw error_1.EXIT_ON_ERROR;
    }
    const [bundleDefault, bundleCjsProd, bundleUmdDev, bundleUmdProd,] = await bundles;
    // write rollup bundles
    const writeRollupBundlesMessage = `Writing esm, cjs${addUmdBuild ? ', umd' : ''} builds`;
    try {
        const output = config_1.writeBundles({
            cwd,
            kebabCasePkgName,
            bundleDefault,
            bundleCjsProd,
            bundleUmdDev,
            bundleUmdProd,
            outputPluginsDefault,
            outputPluginsProduction,
            umdNameForPkg,
            umdDependencyGlobals,
        });
        await utils_1.progressEstimator(output, writeRollupBundlesMessage, {
            id: `${kebabCasePkgName}-${writeRollupBundlesMessage}`,
        });
    }
    catch (error) {
        error_1.logError({
            failedAt: writeRollupBundlesMessage,
            message: error_1.errorAsObjectWithMessage(error).message,
            fullError: error,
        });
        throw error_1.EXIT_ON_ERROR;
    }
    // build success
    await utils_1.progressEstimator(Promise.resolve(), 'BUILD SUCCESS 😁😘');
};
const errorExitCode = process.argv[2] === 'watch' ? 0 : 1;
mooliRollup().catch((error) => {
    if (error === error_1.EXIT_ON_ERROR) {
        utils_1.cleanDist()
            .catch()
            .finally(() => {
            process.exit(errorExitCode);
        });
    }
    else {
        error_1.logError({ message: 'UNKNOWN ERROR', fullError: error });
        process.exit(errorExitCode);
    }
});
//# sourceMappingURL=index.js.map